@@ -43,12 +43,11 @@ class GActions
 {
 public:
 
-  GActions(const std::shared_ptr<Model>& model,
-           const std::shared_ptr<Controller>& controller):
+  explicit GActions(const std::shared_ptr<Controller>& controller):
     m_action_group(g_simple_action_group_new()),
     m_controller(controller)
   {
-    set_model(model);
+    set_model(controller->get_model());
 
     const GActionEntry entries[] = {
         { "activate-transfer", on_tap, "s", nullptr },
@@ -75,7 +74,7 @@ public:
 
   }
 
-  void set_model(const std::shared_ptr<Model>& model)
+  void set_model(const std::shared_ptr<const Model>& model)
   {
     // out with the old...
     auto& c = m_connections;
@@ -161,6 +160,7 @@ private:
           }
 
         g_variant_builder_add(&b, "{sv}", "state", g_variant_new_int32(transfer->state));
+        g_variant_builder_add(&b, "{sv}", "state-label", g_variant_new_string(transfer->custom_state.c_str()));
       }
     else
       {
@@ -252,7 +252,7 @@ private:
   }
 
   GSimpleActionGroup* m_action_group = nullptr;
-  std::shared_ptr<Model> m_model;
+  std::shared_ptr<const Model> m_model;
   std::shared_ptr<Controller> m_controller;
   std::set<core::ScopedConnection> m_connections;
 
@@ -279,7 +279,7 @@ public:
   GMenuModel* menu_model() { return G_MENU_MODEL(m_menu); }
 
   Menu(const char* name_in,
-       const std::shared_ptr<Model>& model,
+       const std::shared_ptr<const Model>& model,
        const std::shared_ptr<GActions>& gactions):
     m_name{name_in},
     m_gactions{gactions}
@@ -291,12 +291,17 @@ public:
 
   virtual ~Menu()
   {
-    if (m_update_header_tag)
-      g_source_remove(m_update_header_tag);
+    if (m_update_header_tag > 0)
+      {
+        g_source_remove(m_update_header_tag);
+        // commit any pending change on header
+        update_header();
+      }
+
     g_clear_object(&m_menu);
   }
 
-  void set_model (const std::shared_ptr<Model>& model)
+  void set_model (const std::shared_ptr<const Model>& model)
   {
     auto& c = m_connections;
     c.clear();
@@ -377,8 +382,9 @@ private:
     int n_failed = 0;
     int n_paused = 0;
 
-    for (const auto& transfer : m_model->get_all())
+    for (auto it=m_visible_transfers.cbegin(); it!=m_visible_transfers.cend(); ++it)
       {
+        auto transfer = m_model->get((*it).first);
         switch (transfer->state)
           {
             case Transfer::RUNNING:
@@ -423,10 +429,12 @@ private:
      currently incomplete because they're either ongoing or paused. */
   bool header_should_be_visible() const
   {
-    for (const auto& transfer : m_model->get_all())
-      if (transfer->state != Transfer::FINISHED)
-        return true;
-
+    for (auto it=m_visible_transfers.cbegin(); it!=m_visible_transfers.cend(); ++it)
+      {
+        auto transfer = m_model->get((*it).first);
+        if (transfer->state != Transfer::FINISHED)
+          return true;
+      }
     return false;
   }
 
@@ -563,7 +571,7 @@ private:
     return item;
   }
 
-  static bool bulk_menu_item_is_equal(GMenuModel* model, 
+  static bool bulk_menu_item_is_equal(GMenuModel* model,
                                       int pos,
                                       GMenuItem* item)
   {
@@ -624,7 +632,6 @@ private:
 
     g_menu_item_set_attribute (menu_item, ATTRIBUTE_X_TYPE,
                                "s", "com.canonical.indicator.transfer");
-
     GVariant * serialized_icon = nullptr;
     if (!t->app_icon.empty() && g_file_test(t->app_icon.c_str(), G_FILE_TEST_EXISTS))
       {
@@ -693,6 +700,19 @@ private:
   void update(const Transfer::Id& id)
   {
     const auto t = m_model->get(id);
+
+    // For now we do not want to keep canceled or error downloads on the list
+    // the app will handle it internally
+    switch (t->state)
+      {
+        case Transfer::CANCELED:
+        case Transfer::ERROR:
+          remove(id);
+          return;
+        default:
+          break;
+      }
+
     g_return_if_fail(t);
 
     // if the transfer already has a menu item, find it
@@ -772,7 +792,7 @@ private:
   GMenu* m_menu = nullptr;
   const char* const m_name;
 
-  std::shared_ptr<Model> m_model;
+  std::shared_ptr<const Model> m_model;
   std::shared_ptr<GActions> m_gactions;
   std::map<Transfer::Id,Section> m_visible_transfers;
   GMenu* m_submenu = nullptr;
@@ -936,17 +956,16 @@ class GMenuView::Impl
 {
 public:
 
-  Impl (const std::shared_ptr<Model>& model,
-        const std::shared_ptr<Controller>& controller):
-    m_model(model),
+  explicit Impl (const std::shared_ptr<Controller>& controller):
     m_controller(controller),
-    m_gactions(new GActions(model, controller)),
+    m_gactions(new GActions(controller)),
     m_exporter(new Exporter)
   {
+    set_model(controller->get_model());
     // create the Menus
     for(int i=0; i<Menu::NUM_PROFILES; i++)
       m_menus.push_back(create_menu_for_profile(Menu::Profile(i)));
-  
+
     m_exporter->publish(m_gactions, m_menus);
   }
 
@@ -954,7 +973,7 @@ public:
   {
   }
 
-  void set_model(const std::shared_ptr<Model>& model)
+  void set_model(const std::shared_ptr<const Model>& model)
   {
     m_model = model;
 
@@ -976,7 +995,7 @@ private:
     return m;
   }
 
-  std::shared_ptr<Model> m_model;
+  std::shared_ptr<const Model> m_model;
   std::shared_ptr<Controller> m_controller;
   std::shared_ptr<GActions> m_gactions;
   std::vector<std::shared_ptr<Menu>> m_menus;
@@ -987,9 +1006,8 @@ private:
 ****
 ***/
 
-GMenuView::GMenuView(const std::shared_ptr<Model>& model,
-                 const std::shared_ptr<Controller>& controller):
-  p(new Impl(model, controller))
+GMenuView::GMenuView(const std::shared_ptr<Controller>& controller):
+  p(new Impl(controller))
 {
 }
 
@@ -997,11 +1015,6 @@ GMenuView::~GMenuView()
 {
 }
 
-void GMenuView::set_model(const std::shared_ptr<Model>& model)
-{
-  p->set_model(model);
-}
-
 const core::Signal<>& GMenuView::name_lost() const
 {
   return p->name_lost();
